# Copyright (c) 2018 Evalf
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import typing, typing_extensions, enum

class Level(enum.Enum):

  debug = 0
  info = 1
  user = 2
  warning = 3
  error = 4

class IO(typing_extensions.Protocol[typing.AnyStr]):

  def fileno(self) -> int: ...
  def readable(self) -> bool: ...
  def read(self, n: int = ...) -> typing.AnyStr: ...
  def writable(self) -> bool: ...
  def write(self, data: typing.AnyStr) -> int: ...
  def seekable(self) -> bool: ...
  def seek(self, offset: int, whence: int = ...) -> int: ...

class Log(typing_extensions.Protocol):

  def pushcontext(self, title: str) -> None: ...
  def popcontext(self) -> None: ...
  def recontext(self, title: str) -> None: ...
  def write(self, text: str, level: Level) -> None: ...
  def open(self, filename: str, mode: str, level: Level) -> typing_extensions.ContextManager[IO[typing.Any]]: ...

# vim:sw=2:sts=2:et
